#include <qstring.h>
#include <qstringlist.h>
#include <qtextstream.h>
#include <qfile.h>

#ifndef __CGenericConfig__H
#define __CGenericConfig__H

/*
	CGenericConfig.h	TimeTrack V1.0
	@Author: Thomas Thiel
	@EMail: thielt@cs.uni-bonn.de
	@Created: 20010623
	
	This class is GPL Code. Feel free to distribute, change and burn
	this code, but always burn the authors name with it !
*/

/* 
	CGenericConfig: Klasse zur Speicherung generischer 
			Optionsstrings
			
	Diese Klasse erzeugt aus QString-Objekten Optionspaare
	zum Festlegen von Optionen persistenter Art. Die Benutzung 
	ist denkbar einfach: Konstruktion eines Objektes mit Angabe 
	eines Dateinamens, auslesen der Daten wird im konstruktor 
	automatisch abgewickelt, kann aber auch manuell wiederholt 
	werden durch Aufruf von readConfig(). Der Rest sollte aus der
	Beschreibung der Objektmethoden selbsterklrend sein.  
*/

class CGenericConfig
{

public:

  // Konstruktor und Destruktor
  CGenericConfig (QString filename);
  ~CGenericConfig ();

  // Schreibt alle existierenden Optionspaare in die Datei
  bool writeConfig ();

  // Liest erneut alle Optionspaare ein und speichert sie intern.
  bool readConfig ();

  // Fgt ein Optionspaar hinzu oder ndert ein bereits bestehendes 
  // Paar. Welches Verhalten gewhlt wird, entscheidet die Funktion 
  // automatisch, je nachdem ob der Identifier bereits existiert.
  bool addValue (QString id, QString val);

  // Entfernt ein Optionspaar. Gibt Erfolg oder Mierfolg zurck.
  // (false, wenn identifier nicht existiert hat)
  bool remValue (QString id);

  // Gibt einen Optionswert aus und stellt diesen als String dar.
  QString getValue (QString id);

  // dito, allerdings Ausgabe in ein int.
  int getIntValue (QString id);
  /** Prft auf Vorhandensein eines Tupels */
  bool existValue (QString label);

protected:

  // Filehandle und Textstream Objekt zur Dateiverwaltung 
    QFile * fh;
  QTextStream *ts;

  // Die Listen von Identifiern und dazugehrigen Werten.
  QStringList identifiers;
  QStringList values;


};

#endif
