/***************************************************************************
 *   Copyright (C) 2004 by Thomas Thiel                                    *
 *   wolfchild@wolfchild.de                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include <qlistview.h>

#include "cimportselection.h"

// Secondary constructor - UNUSED
// TODO:: OUT ! OUT I SAY !
CImportSelection::CImportSelection(QWidget* parent, const char* name, bool modal, WFlags fl)
        : ImportSelection(parent,name, modal,fl)
{
}

// Main constructor. Fills the central ListView with the Entries from the inp_domnode.
CImportSelection::CImportSelection(QDomNode inp_domNode, QWidget* parent, const char* name, bool modal, WFlags fl)
        : ImportSelection(parent,name, modal,fl)
{
	// initialize the attributes
	QListView *lvw_selection=getTimerListView();
	domNode=inp_domNode;
	
	// get all CTimerWidgets from the DOM Subtree
	domList=domNode.toElement().elementsByTagName("CTimerWidget");
	
	// for all of them do:
	for( unsigned int i=0; i<domList.count();i++ )
	{
		QString parser;

		// convert it to an element
		QDomElement domElement=domList.item(i).toElement();
		
		// parse Description and Name
		QDomNodeList domListDesc = domElement.elementsByTagName("TimerDescription");
		QDomElement momElement = domListDesc.item(0).toElement();
		
		// Spawn new QCheckListItem for the entry
		QCheckListItem *momItem = new QCheckListItem(lvw_selection, domList.item(i).toElement().attribute("caption","unnamed Timer"), QCheckListItem::CheckBox);
		momItem->setText(1,momElement.firstChild().toText().data());
		momItem->setText(2,QString::number(i));

	};
}

CImportSelection::~CImportSelection()
{
}

// returns the filtered DOM SubTree
QDomNode CImportSelection::getFilteredDomNode()
{
	QListView *lvw_selection=getTimerListView();
	QCheckListItem *momItem=NULL;
	QDomNode newNode;
	QDomNode momNode;
		
	// iterate all the QCheckListItems
	QListViewItemIterator it( lvw_selection );
  while ( it.current() ) 
	{
    momItem = (QCheckListItem*)it.current();
    
		// if the CheckBox isn't checked...
		if (momItem->state()==QCheckListItem::Off)
		{
			// get the corresponding Node and change the TagName, So the parser of Ctimerwidget wont recognize it as a CTimerWidget
			momNode=domList.item(momItem->text(2).toInt());
			momNode.toElement().setTagName("CTimerWidgetIGNORED");
		};
		
    ++it;
  };
	
	// return the filtered Subtree
	return domNode;
};

/*$SPECIALIZATION$*/


