/*************************************************************************** 
 *   Copyright (C) 2004 by Thomas Thiel                                    *
 *   wolfchild@wolfchild.de                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qlayout.h>
#include <qsplitter.h>
#include <qpushbutton.h>
#include <qinputdialog.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qdom.h>
#include <qobject.h>
#include <qobjectlist.h>
#include <qpopupmenu.h>
#include <qmessagebox.h>
#include <qmenubar.h>
#include <qfiledialog.h>
#include <qtimer.h>
#include <qscrollview.h>

#include <iostream>

using std::cout;
using std::cin;
using std::cerr;
using std::clog;
using std::endl;

#include "cimportselection.h"
#include "cexportselection.h"
#include "cmainwindow.h"
#include "ctimerwidget.h" 
#include "timetrack_images.h"
#include "caboutdialog.h"
#include "CGenericConfig.h"
#include "cconfigdialog.h"
#include "creportgenerator.h"

extern CGenericConfig *TimeTrackConf;

// Main constructor. Prepares the MainWindow Framework.
CMainWindow::CMainWindow(QWidget *parent, const char *name)
 : QMainWindow(parent, name)
{
	// set the window size to 600x400
	setGeometry(x(),y(),600,400);

	// All toolbars should use TextLabels in this Program
	setUsesTextLabel(false);
	
	//
	// the following Block prepares the QActions for the menus, sets the necessary connections and their IconSets.
	//
	
	img_newtimer = new QPixmap();
	img_newtimer->convertFromImage(qembed_findImage("newtimer"),0);
	newtimer = new QAction( "New Timer", QIconSet( *img_newtimer, *img_newtimer ), "&New Timer", CTRL+Key_N, this, "newtimer" );
        connect( newtimer, SIGNAL( activated() ) , this, SLOT( spawnnewcounter() ) );
	
	img_newtab = new QPixmap();
	img_newtab->convertFromImage(qembed_findImage("newtab"),0);
	newtab = new QAction( "New Tab...", QIconSet( *img_newtab, *img_newtab ), "&New Tab...", CTRL+Key_T, this, "newtab" );
	      connect( newtab, SIGNAL( activated() ) , this, SLOT( spawnnewtab() ) );
	
	img_filesave = new QPixmap();
	img_filesave->convertFromImage(qembed_findImage("filesave"),0);
	filesave = new QAction( "Save", QIconSet( *img_filesave, *img_filesave ), "&Save",CTRL+Key_S, this, "save" );
        connect( filesave, SIGNAL( activated() ) , this, SLOT( saveCurrent() ) );
	
	filesaveas = new QAction( "Save as...", QIconSet( *img_filesave, *img_filesave ), "&Save as...",CTRL+Key_S, this, "saveas" );
        connect( filesaveas, SIGNAL( activated() ) , this, SLOT( saveAs() ) );

	img_fileload = new QPixmap();
	img_fileload->convertFromImage(qembed_findImage("fileopen"),0);
	fileload = new QAction( "Load...", QIconSet( *img_fileload, *img_fileload ), "&Load...",CTRL+Key_L, this, "load" );
        connect( fileload, SIGNAL( activated() ) , this, SLOT( load() ) );
	
	fileimport = new QAction( "Import Timers...", QIconSet( *img_fileload, *img_fileload ), "&Import Timers...",CTRL+Key_I, this, "importtimers" );
        connect( fileimport, SIGNAL( activated() ) , this, SLOT( insertFile() ) );
	
	fileexport = new QAction( "Export Timers...", QIconSet( *img_filesave, *img_filesave ), "&Export Timers...",CTRL+Key_I, this, "exporttimers" );
        connect( fileexport, SIGNAL( activated() ) , this, SLOT( exportFile() ) );

	img_clearall = new QPixmap();
	img_clearall->convertFromImage(qembed_findImage("clearall"),0);
	clearall = new QAction( "Close all", QIconSet( *img_clearall, *img_clearall ), "C&lose all tabs",CTRL+Key_X, this, "closeall" );
        connect( clearall, SIGNAL( activated() ) , this, SLOT( clearAll() ) );
	
	clearcurrent = new QAction( "Close current", QIconSet( *img_clearall, *img_clearall ), "&Close current tab",CTRL+Key_C, this, "closecurrent" );
        connect( clearcurrent, SIGNAL( activated() ) , this, SLOT( clearCurrent() ) );
	
	img_renametab = new QPixmap();
	img_renametab->convertFromImage(qembed_findImage("rename"),0);
	renametab = new QAction( "Rename Tab...", QIconSet( *img_renametab, *img_renametab ), "&Rename Tab...", CTRL+Key_R, this, "renametab");
        connect( renametab, SIGNAL( activated() ), this, SLOT( renameTab() ) );
	
	about = new QAction("About TimeTrack...", "&About TimeTrack", CTRL+Key_A, this, "about");
	      connect( about, SIGNAL( activated() ), this, SLOT( showAbout() ) );

        report_tab = new QAction("Generate Report (this tab)", QIconSet( *img_filesave, *img_filesave ), "&Generate Report (this tab)", CTRL+Key_G, this, "report_tab");
	connect( report_tab, SIGNAL( activated() ), this, SLOT( reportTab() ) );
        
	report_all = new QAction("Generate Report (all tabs)", QIconSet( *img_filesave, *img_filesave ), "G&enerate Report (all tabs)", CTRL+Key_E, this, "report_all");
	connect( report_all, SIGNAL( activated() ), this, SLOT( reportAll() ) );
	      	
	options = new QAction("Configuration...", "&Configuration", CTRL+Key_A, this, "configuration");
	      connect( options, SIGNAL( activated() ), this, SLOT( showOptions() ) );
	
	img_quit = new QPixmap();
	img_quit->convertFromImage(qembed_findImage("exit"),0);
	quit = new QAction("Quit", QIconSet( *img_quit, *img_quit ), "&Quit", CTRL+Key_Q, this, "quit");
	      connect( quit, SIGNAL( activated() ), this, SLOT( close() ) );
	
	// this prepares the Menubar
	file = new QPopupMenu( this );
	menuBar()->insertItem( "&File", file);
	menuBar()->insertSeparator();
	help = new QPopupMenu( this );
	menuBar()->insertItem( "&Help", help);

	// preparing the quick-toolbar
	tools = new QToolBar( this, "main operations" );
	tools->setOrientation(Qt::Vertical);
	moveDockWindow(tools,Qt::DockLeft);
	tools->setLabel( "TimeTrack Toolbox");

	// preparing the file toolbar
	filetools = new QToolBar( this, "file operations" );
	filetools->setOrientation(Qt::Vertical);
	moveDockWindow(filetools,Qt::DockLeft);
	filetools->setLabel( "TimeTrack FileBox");
	
	// here the actions are arranged into the file toolbox
	fileload->addTo(filetools);
	filesave->addTo(filetools);
	filetools->addSeparator();
	quit->addTo(filetools);

	// Here the Actions are added to the file menu...
	newtimer->addTo( file );
	newtab->addTo( file );
	file->insertSeparator();
	fileload->addTo( file );
	filesave->addTo( file );
	filesaveas->addTo( file );
	file->insertSeparator();
	fileexport->addTo( file );
	fileimport->addTo( file );
	file->insertSeparator();
	report_tab->addTo(file);
	report_all->addTo(file);
	file->insertSeparator();
	renametab->addTo( file );
	clearcurrent->addTo( file );
	clearall->addTo( file );
	file->insertSeparator();
	options->addTo( file );
	file->insertSeparator();
	quit->addTo( file );
		
	// ... the help menu ...
	about->addTo( help );
	
	// and the tools-toolbar.
	newtimer->addTo( tools );
	newtab->addTo( tools );
	tools->addSeparator();
	renametab->addTo( tools );
	clearall->addTo( tools );
	
	// initialize the rest...
	currentFile="";

	filesave->setEnabled(false);
	filesaveas->setEnabled(false);
}


CMainWindow::~CMainWindow()
{
	// Qt does all the dirty handywork for us, so this is left intentionally  blank.
}




/*!
    \fn CMainWindow::init()
		this function prepares all the stuff that needs a prepared QMainWindow to work. It has to be called 
		directly after construction.
 */
void CMainWindow::init()
{
	// add the main widget, a TabView. In each tabview there is a QSplitter that manages our CTimerWidgets
	tabwidget = new QTabWidget(this);
	setCentralWidget(tabwidget);
	
	QScrollView* sv = new QScrollView(tabwidget);
	QSplitter *momSplitter=new QSplitter(sv->viewport());
	
	sv->addChild((QWidget*)momSplitter);
	
	tabwidget->addTab((QWidget*)sv,"First Tab");
	tabwidget->show();
	
	sv->show();
		
	momSplitter->setOrientation(QSplitter::Vertical);
	momSplitter->setHandleWidth(0);

	if (!TimeTrackConf->existValue("TT_DescMode")) TimeTrackConf->addValue("TT_DescMode","end");
		
	if (TimeTrackConf->existValue("TT_AOFilename"))
	{
		QString filename = TimeTrackConf->getValue("TT_AOFilename");
		
		if (filename!="") loadFile(filename);
	};
	
	TimeTrackConf->writeConfig();
};

// called to spawn the SplashScreen
void CMainWindow::showSplash()
{	
	splash_image = new QPixmap();
	splash_image->convertFromImage(qembed_findImage("timetrack"));
	splash = new QSplashScreen(*splash_image, Qt::WStyle_Splash);
	splash->show();
	// after 3 secs, kill the splash
	QTimer::singleShot(3000, this, SLOT( hideSplash() ) );
}


/*!
    \fn CMainWindow::spawnnewtab()
		called if a new Tab has to be added to the tabview. Spawns not only a new tab but also the necessary QSplitter for 
		CTimerWidget management.
 */
void CMainWindow::spawnnewtab()
{
	// ask for the name of the new Tab.
	QString label = QInputDialog::getText("TimeTrack", "Enter the name of your new tab:", QLineEdit::Normal, "New Tab", 0, this );
	
	tabwidget->hide();
	
	
	QScrollView* sv = new QScrollView(tabwidget);
	QSplitter *momSplitter=new QSplitter(sv->viewport());
	
	momSplitter->setOrientation(QSplitter::Vertical);
	momSplitter->setHandleWidth(0);
	
	sv->addChild((QWidget*)momSplitter);
	
	tabwidget->addTab((QWidget*)sv,label);

	sv->show();

	tabwidget->show();
};


/*!
    \fn CMainWindow::spawnnewcounter()
		called if the User wants a new Timer
 */
void CMainWindow::spawnnewcounter()
{
	QSplitter *momSplitter=(QSplitter*)getSplitter(((QScrollView*)(tabwidget->currentPage()))->viewport());
	if (momSplitter)
	{
		// allocate a new CTimerWidget and put it into the current tabpage.
		CTimerWidget *momTimer;
		momTimer = new CTimerWidget(this,momSplitter);
		momTimer->tabname=tabwidget->label(tabwidget->currentPageIndex());
		momTimer->show();
	}
	else
	{
		cerr << "CMainWindow::spawnnewcounter(): Splitter could not be found !" << endl;
	};
}


// loadFile() is called if the User wants to load his timers from a file
void CMainWindow::load()
{
	// get Filename and check it
	QString filename=QFileDialog::getOpenFileName(
                    	".",
                    	"XML-Files (*.xml)",
                    	this,
                    	"TimeTrack File dialog",
                    	"Choose a file to open" );
			
	if (filename!="") loadFile(filename);
};


void CMainWindow::loadFile(const QString & filename)
{
	if (filename=="") return;
	
	// prepare a QDomDocument and read it from a file
	QDomDocument doc( "XML" );
    	QFile inp_file( filename );
    	
	if ( !inp_file.open( IO_ReadOnly ) )
	{
		cerr << "loadfile could not open file" << endl;
	        return;
	};
	
	// the xml parser wil put error data into these vars.
	QString errorMsg;
	int errorLine,errorCol;
    	
	if ( !doc.setContent( &inp_file, &errorMsg, &errorLine, &errorCol) ) 
	{
		inp_file.close();
		cerr << "loadfile could not parse xml\nReason (Row:" << errorLine << "/Col:" << errorCol << "): \"" << errorMsg << "\"" << endl;
	  return;
	};

	// the file has been parsed. Close it.
	inp_file.close();

	// As we don't need the old CTimerWidgets anymore, get rid of them all.
	// clearAll();	
	tabwidget->hide();
	delete tabwidget;
	
	// allocate a new tabwidget and prepare it for the CTimerWidgets.
	tabwidget = new QTabWidget(this);
	setCentralWidget(tabwidget);
	
	// get the root Node
	QDomElement root = doc.documentElement();
	
	// check if we have the correct file
	if ((root.tagName()=="TimeTrack")&&(root.attribute("fileformat","")=="1"))
	{
		// obtain all CTimerWidget Data from the DOM Tree...
		QDomNodeList domList = root.elementsByTagName("CTimerWidget");

		//... and for all of them do this:
		for( unsigned int i=0; i<domList.count();i++ )
		{
			CTimerWidget *momTimer;
			QDomElement momElement=domList.item(i).toElement();
			
			int tabindex=0;
			bool tabfound=false;
			
			// find out, if the tabname of the counter is already there
			for (int i=0; i<tabwidget->count(); i++)
			{
				if (tabwidget->label(i)==momElement.attribute("tabname","New Tab")) 
				{
					tabindex=i;
					tabfound=true;
				}
			};
			
			// if so, spawn the new CTimerWidget and let it parse his config.
			if (tabfound)
			{
				momTimer = new CTimerWidget(this,(QSplitter*)getSplitter(((QScrollView*)(tabwidget->page(tabindex)))->viewport()));
				momTimer->tabname=tabwidget->label(tabindex);
				momTimer->readTimerData(momElement);
				((QScrollView*)(tabwidget->page(tabindex)))->show();
				momTimer->show();
			}
			// if not, allocate the missing tab,spawn the new CTimerWidget and let it parse his config.
			else
			{
				QScrollView* sv = new QScrollView(tabwidget);
				QSplitter *momSplitter=new QSplitter(sv->viewport());
				
				momSplitter->setOrientation(QSplitter::Vertical);
				momSplitter->setHandleWidth(0);
					
				sv->addChild((QWidget*)momSplitter);

				momSplitter->show();
				
				sv->show();				

				tabwidget->addTab((QWidget*)sv,momElement.attribute("tabname","New Tab"));
				tabwidget->show();
			
				momTimer = new CTimerWidget(this,momSplitter);
				momTimer->tabname=momElement.attribute("tabname","New Tab");
				momTimer->readTimerData(momElement);
				momTimer->show();
				
			}
			
		};

	}
	
	tabwidget->show();
  	tabwidget->setCurrentPage(0);
	
	currentFile=filename;
        setCaption("TimeTrack - "+currentFile);

	filesave->setEnabled(false);
	filesaveas->setEnabled(true);
};


// Same as loadFile, but the User can select the Timers that are to be merged 
// with the existing Timers.
void CMainWindow::insertFile()
{
	// get Filename and check it
	QString filename=QFileDialog::getOpenFileName(
                    	".",
                    	"XML-Files (*.xml)",
                    	this,
                    	"TimeTrack File dialog",
                    	"Choose a file to insert" );

	if (filename=="") return;
	
	// prepare a QDomDocument and read it from a file
	QDomDocument doc( "XML" );
    	QFile inp_file( filename );
    	
	if ( !inp_file.open( IO_ReadOnly ) )
	{
		cerr << "loadfile could not open file" << endl;
	        return;
	};
	
	// the xml parser wil put error data into these vars.
	QString errorMsg;
	int errorLine,errorCol;
    	
	if ( !doc.setContent( &inp_file, &errorMsg, &errorLine, &errorCol) ) 
	{
	        inp_file.close();
		cerr << "loadfile could not parse xml\nReason (Row:" << errorLine << "/Col:" << errorCol << "): \"" << errorMsg << "\"" << endl;
	        return;
    	};

	// the file has been parsed. Close it.
	inp_file.close();

	// as we still want the other Ctimerwidgets, we don't delete them here, like in loadFile()
	tabwidget->hide();
	
	
	// get the root Node
	QDomElement root = doc.documentElement();
	
	// check if we have the correct file
	if ((root.tagName()=="TimeTrack")&&(root.attribute("fileformat","")=="1"))
	{
		// obtain all CTimerWidget Data from the DOM Tree...
		QDomNodeList domList = root.elementsByTagName("CTimerWidget");
		// initialize filteredDomNode with root, so that if the user cancels the CImport selection, the filteredDomNode is the 
		// unfiltered root node.
		QDomNode filteredDomNode=root;
		
		// spawn the CImportSelection and let it filter the unwanted CTimerwidget Nodes.
		CImportSelection *importSelection = new CImportSelection(root, this);
		if (importSelection->exec()==QDialog::Accepted) 
		{
			// if the Dialog was accepted, retrieve the filtered Node.
			filteredDomNode=importSelection->getFilteredDomNode();
			delete importSelection;
		}
		else
		{
			delete importSelection;
			return;
		};
		
		// obtain all CTimerWidget Data from the filtered DOM Tree...
		domList = filteredDomNode.toElement().elementsByTagName("CTimerWidget");
		
		// ... and for all of them do this:
		for( unsigned int i=0; i<domList.count();i++ )
		{
			CTimerWidget *momTimer;
			QDomElement momElement=domList.item(i).toElement();
			
			int tabindex=0;
			bool tabfound=false;
			
			// find out, if the tabname of the counter is already there
			for (int i=0; i<tabwidget->count(); i++)
			{
				if (tabwidget->label(i)==momElement.attribute("tabname","New Tab")) 
				{
					tabindex=i;
					tabfound=true;
				}
			};
			
			// if so, spawn the new CTimerWidget and let it parse his config.
			if (tabfound)
			{
				momTimer = new
				CTimerWidget(getSplitter(((QScrollView*)(tabwidget->page(tabindex)))->viewport()));
				momTimer->tabname=tabwidget->label(tabindex);
				momTimer->readTimerData(momElement);
				momTimer->show();
			}
			// if not, allocate the missing tab,spawn the new CTimerWidget and let it parse his config.
			else
			{
				QScrollView* sv = new QScrollView(tabwidget);
				QSplitter *momSplitter=new QSplitter(sv->viewport());
				
				momSplitter->setOrientation(QSplitter::Vertical);
				momSplitter->setHandleWidth(0);
				momSplitter->show();
				
				tabwidget->addTab((QWidget*)sv,momElement.attribute("tabname","New Tab"));
				
				sv->show();
			
				momTimer = new CTimerWidget(this,momSplitter);
				momTimer->tabname=momElement.attribute("tabname","New Tab");
				momTimer->readTimerData(momElement);
				momTimer->show();
				
				
			}
			
		};

	}

	tabwidget->show();

        setCaption("TimeTrack - * "+currentFile);

	filesave->setEnabled(true);
	filesaveas->setEnabled(true);
};

void CMainWindow::saveAs()
{
	QString filename=QFileDialog::getSaveFileName(
                    currentFile,
                    "XML-Files (*.xml)",
                    this,
                    "TimeTrack File dialog",
                    "Choose a file to save into" );
	
	if (filename!="") saveFile(filename);
	    
};

void CMainWindow::saveCurrent()
{
	if (currentFile=="") saveAs();
	else saveFile(currentFile);
};

// called, if the user wants to save the current timer setup
void CMainWindow::saveFile(const QString & filename)
{
	if (filename=="") return;
	
	// prepare a file to save the xml data in.
	QFile outp_file( filename );
	
	// prepare a new DOM Document.
	QDomDocument doc( "XML" );
	QDomElement root = doc.createElement( "TimeTrack" );
	root.setAttribute("fileformat","1");
	doc.appendChild( root );

	// Now traverse all Tabs...
	for( int i=0; i<tabwidget->count(); i++)
	{
		// ..retrieve the children of each of them...
		QObjectList *objList=getSplitter(((QScrollView*)(tabwidget->page(i)))->viewport())->queryList("CTimerWidget",0,false,false);
		
		QObject *momEntry=NULL;
		
		// and for all CTimerWidgets (which are the grandchildren of the tabpage) store the data
		// in the document.
		for (  momEntry = objList->first(); momEntry; momEntry = objList->next() )
		{
			if (momEntry)
			{
				QDomElement dom_twidget = ((CTimerWidget*)(momEntry))->writeTimerData();
				root.appendChild(doc.importNode(dom_twidget,true));
			}
		};
		
	};
	
	// Then store the document in the file and close it.
	if ( outp_file.open( IO_WriteOnly ) ) 
	{
		QTextStream txtstream( &outp_file );
		txtstream << doc.toString() << "\n";
		outp_file.close();
	};
	
	currentFile=filename;
        setCaption("TimeTrack - "+currentFile);

	filesaveas->setEnabled(true);
	filesave->setEnabled(false);

	

};


// called, if the user wants to export some Timers from the current setup
void CMainWindow::exportFile()
{
	// get the filename.
	QString filename=QFileDialog::getSaveFileName(
                    currentFile,
                    "XML-Files (*.xml)",
                    this,
                    "TimeTrack File dialog",
                    "Choose a file to export into" );
	
	if (filename=="") return;
	
	// prepare a file to save the xml data in.
	QFile outp_file( filename );
	
	// prepare a new DOM Document.
	QDomDocument doc( "XML" );
	QDomElement root = doc.createElement( "TimeTrack" );
	root.setAttribute("fileformat","1");
	doc.appendChild( root );

	// Now traverse all Tabs...
	for( int i=0; i<tabwidget->count(); i++)
	{
		// ..retrieve the children of each of them...
		QObjectList *objList=getSplitter(((QScrollView*)(tabwidget->page(i)))->viewport())->queryList("CTimerWidget",0,false,false);
		
		QObject *momEntry=NULL;
		
		// and for all CTimerWidgets (which are the grandchildren of the tabpage) store the data
		// in the document.
		for (  momEntry = objList->first(); momEntry; momEntry = objList->next() )
		{
			if (momEntry)
			{
				QDomElement dom_twidget = ((CTimerWidget*)(momEntry))->writeTimerData();
				root.appendChild(doc.importNode(dom_twidget,true));
			}
		};
		
	};
		
	// initialize filteredDomNode with root, so that if the user cancels the CImport selection, the filteredDomNode is the 
	// unfiltered root node.
	QDomNode filteredDomNode=root;
	
	// spawn the CImportSelection and let it filter the unwanted CTimerwidget Nodes.
	CExportSelection *exportSelection = new CExportSelection(root, this);
	if (exportSelection->exec()==QDialog::Accepted) 
	{
		// if the Dialog was accepted, retrieve the filtered Node.
		filteredDomNode=exportSelection->getFilteredDomNode();
		delete exportSelection;
	}
	else
	{
		delete exportSelection;
		return;
	};
	
	// now obtain all ignored CTimerWidget Data from the DOM Tree...
	QDomNodeList domList = filteredDomNode.toElement().elementsByTagName("CTimerWidgetIGNORED");
		
	// ... and for all of them do this:
	for( unsigned int i=0; i<domList.count();i++ )
	{
		// kill'em all... what a bloodshed.
		filteredDomNode.removeChild(domList.item(i));
	};
	
	// replace root with filteredDomNode;
	doc.appendChild(doc.importNode(filteredDomNode,true));
	// and remove the original root node.
	doc.removeChild(root);

	// then store the document in the file and close it.
	if ( outp_file.open( IO_WriteOnly ) ) 
	{
		QTextStream txtstream( &outp_file );
		txtstream << doc.toString() << "\n";
		outp_file.close();
	};

};

// called if the User wants to delete all Ctimerwidgets and TabPages.
void CMainWindow::clearAll()
{
	// Hide and kill the Tabwidget. The CTimerWidgets are also children of tabwidget, so they get deleted, too.
	// doing it manually would be a pain in the ass, Glory to Qt ! :)
	tabwidget->hide();
	delete tabwidget;
	
	// allocate a new one and prepare it for the CTimerWidgets.
	tabwidget = new QTabWidget(this);
	setCentralWidget(tabwidget);
	
	QScrollView* sv = new QScrollView(tabwidget);
	QSplitter *momSplitter=new QSplitter(sv->viewport());
	
	sv->addChild((QWidget*)momSplitter);
	
	tabwidget->addTab((QWidget*)sv,"First Tab");
	tabwidget->show();
	
	sv->show();
	
	momSplitter->setOrientation(QSplitter::Vertical);
	momSplitter->setHandleWidth(0);
};

// called if the User wants to delete the current tab.
void CMainWindow::clearCurrent()
{
	// get a reference to the current page. remove and delete it.
	QWidget *w = tabwidget->currentPage();
	tabwidget->removePage(w);
	delete w;
	tabwidget->setCurrentPage(0);

        setCaption("TimeTrack - * "+currentFile);

	filesave->setEnabled(true);
	filesaveas->setEnabled(true);	
};

// called if the user wants to rename a tab
void CMainWindow::renameTab()
{
	// just spawn a QInputDialog and rename the Tab if succesful.
	bool ok_pressed=false;
	QString newLabel=QInputDialog::getText("TimeTrack", "Enter the new name for the current Tab", QLineEdit::Normal, tabwidget->label(tabwidget->currentPageIndex()), &ok_pressed, this );
	if (ok_pressed) 
	{
		tabwidget->setTabLabel(tabwidget->currentPage(),newLabel);
		
		QObjectList *objList=getSplitter(((QScrollView*)(tabwidget->currentPage()))->viewport())->queryList("CTimerWidget",0,false,false);
		
		QObject *momEntry=NULL;
		
		// for all CTimerWidgets (which are the grandchildren of the tabpage) change the tabname
		for (  momEntry = objList->first(); momEntry; momEntry = objList->next() )
		{
			if (momEntry)
			{
				((CTimerWidget*)(momEntry))->tabname=tabwidget->label(tabwidget->currentPageIndex());
			};
		};
		
        	setCaption("TimeTrack - * "+currentFile);

		filesave->setEnabled(true);
		filesaveas->setEnabled(true);
	};	
};

// called by the single shot Timer that is started by showSplash()
void CMainWindow::hideSplash()
{
	// if the splashscreen still exists, kill it...
	if (splash)
	{
		splash->hide();
		delete splash;
		delete splash_image;
	};
};


void CMainWindow::showAbout()
{
	CAboutDialog *about_dlg= new CAboutDialog(this);
	about_dlg->exec();
	delete about_dlg;
};

void CMainWindow::close()
{
//	int result=QMessageBox::information(this,"TimeTrack Question","If you quit, your Work will be unsaved.\n\nDo you want to save it now ?",QMessageBox::Ok|QMessageBox::Default, QMessageBox::No, QMessageBox::Cancel|QMessageBox::Escape);

//	if (result==QMessageBox::Ok)
//	{
		saveCurrent();
		QMainWindow::close();
/*	};

	if (result==QMessageBox::No)
	{
		QMainWindow::close();
	};
*/

};

void CMainWindow::showOptions()
{
	CConfigDialog *ConfigDialog=new CConfigDialog(this);
	
	if ((TimeTrackConf->existValue("TT_AOFilename"))&(TimeTrackConf->getValue("TT_AOFilename")!=""))
		ConfigDialog->set_aofilename(TimeTrackConf->getValue("TT_AOFilename"));
	
	if ((TimeTrackConf->existValue("TT_AOTemplate"))&(TimeTrackConf->getValue("TT_AOTemplate")!=""))
		ConfigDialog->set_aotemplate(TimeTrackConf->getValue("TT_AOTemplate"));
	
	if (TimeTrackConf->existValue("TT_DescMode"))
		ConfigDialog->set_descMode(TimeTrackConf->getValue("TT_DescMode")=="end");
	else
		ConfigDialog->set_descMode(true);
	
	if (ConfigDialog->exec()==QDialog::Accepted)
	{
		TimeTrackConf->addValue("TT_AOFilename",ConfigDialog->get_aofilename());
		TimeTrackConf->addValue("TT_AOTemplate",ConfigDialog->get_aottemplate());
		if (ConfigDialog->get_descMode()) TimeTrackConf->addValue("TT_DescMode","end");
			else TimeTrackConf->addValue("TT_DescMode","start");
			
		TimeTrackConf->writeConfig();
	};
			
		
};

void CMainWindow::reportTab()
{
	CReportGenerator reportgen(this,"reportgen");
	if (reportgen.exec()==QDialog::Accepted)
	{
		// prepare a string to store the reports to.
		QString html_reports="";
		html_reports+=reportgen.getDocHeader_substituted(time_to_str(duration_all()),currentFile);
	
		// ..retrieve the children of each of them...
		QObjectList *objList=getSplitter(((QScrollView*)(tabwidget->currentPage()))->viewport())->queryList("CTimerWidget",0,false,false);
		QObject *momEntry=NULL;
		
		if (!(objList->isEmpty()))
		html_reports+=reportgen.getTabHeader_substituted(tabwidget->label(tabwidget->currentPageIndex()),time_to_str(duration_tab(tabwidget->currentPageIndex())),time_to_str(duration_all()),currentFile);
	
		// and for all CTimerWidgets (which are the grandchildren of the tabpage) store the data
		// in the document.
		for (  momEntry = objList->first(); momEntry; momEntry = objList->next() )
		{
			if (momEntry) 
			{
				html_reports+=((CTimerWidget*)(momEntry))->report(&reportgen,currentFile,duration_tab(tabwidget->currentPageIndex()),duration_all(),false);
			}
		}

		if (!(objList->isEmpty()))
		html_reports+=reportgen.getTabFooter_substituted(tabwidget->label(tabwidget->currentPageIndex()),time_to_str(duration_tab(tabwidget->currentPageIndex())),time_to_str(duration_all()),currentFile);
	
	
		html_reports+=reportgen.getDocFooter_substituted(time_to_str(duration_all()),currentFile);

	
	
		QString report_filename = QFileDialog::getSaveFileName(
					QString::null,
					"HTML Files (*.htm *.html)",
					this,
					"TimeTrack save report dialog"
					"Choose a filename to save report under" );
					
		QFile file( report_filename );
		if ( file.open( IO_WriteOnly ) ) 
		{
			QTextStream stream( &file );
			stream << html_reports;
			file.close();
			QMessageBox::information(this,"TimeTrack report generator","The report was successfully saved to file \""+report_filename+"\".",QMessageBox::Ok);
		}
		else QMessageBox::critical(this,"TimeTrack report generator","The report could NOT be successfully saved to file \""+report_filename+"\" !",QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
	};
}

void CMainWindow::reportAll()
{
	CReportGenerator reportgen(this,"reportgen");
	if (reportgen.exec()==QDialog::Accepted)
	{
		// prepare a string to store the reports to.
		QString html_reports="";
		html_reports+=reportgen.getDocHeader_substituted(time_to_str(duration_all()),currentFile);
	
		// Now traverse all Tabs...
		for( int i=0; i<tabwidget->count(); i++)
		{
			// ..retrieve the children of each of them...
			QObjectList *objList=getSplitter(((QScrollView*)(tabwidget->page(i)))->viewport())->queryList("CTimerWidget",0,false,false);
			QObject *momEntry=NULL;
			
			if (!(objList->isEmpty()))
			html_reports+=reportgen.getTabHeader_substituted(tabwidget->label(i),time_to_str(duration_tab(i)),time_to_str(duration_all()),currentFile);
		
			// and for all CTimerWidgets (which are the grandchildren of the tabpage) store the data
			// in the document.
			for (  momEntry = objList->first(); momEntry; momEntry = objList->next() )
			{
				if (momEntry) 
				{
					html_reports+=((CTimerWidget*)(momEntry))->report(&reportgen,currentFile,duration_tab(i),duration_all(),false);
				}
			}

			if (!(objList->isEmpty()))
			html_reports+=reportgen.getTabFooter_substituted(tabwidget->label(i),time_to_str(duration_tab(i)),time_to_str(duration_all()),currentFile);
		}
	
		html_reports+=reportgen.getDocFooter_substituted(time_to_str(duration_all()),currentFile);

	
		
		QString report_filename = QFileDialog::getSaveFileName(
					QString::null,
					"HTML Files (*.htm *.html)",
					this,
					"TimeTrack save report dialog"
					"Choose a filename to save report under" );
					
		QFile file( report_filename );
		if ( file.open( IO_WriteOnly ) ) 
		{
			QTextStream stream( &file );
			stream << html_reports;
			file.close();
			QMessageBox::information(this,"TimeTrack report generator","The report was successfully saved to file \""+report_filename+"\".",QMessageBox::Ok);
		}
		else QMessageBox::critical(this,"TimeTrack report generator","The report could NOT be successfully saved to file \""+report_filename+"\" !",QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
	};
};


/*!
    \fn CMainWindow::getSplitter(const QWidget & *tabwidget )
 */
QWidget* CMainWindow::getSplitter(const QWidget *viewport )
{
	// check if viewport is valid pointer
	if (!viewport) return NULL;
	// retrieve the children of the viewport...
	QObjectList *objList=viewport->queryList("QSplitter",0,false,false);
	QObject *momEntry=NULL;

	// and for all QSplitter (which are the Children of the tabpage) store the data
	// in the document.
	momEntry=objList->first();
	
	// if momEntry is valid return it, else return null.
	if (momEntry) return (QWidget*)momEntry;
	return NULL;
}


/*!
    \fn CMainWindow::duration_all()
 */
long int CMainWindow::duration_all()
{
	long int result=0;
	for( int i=0; i<tabwidget->count(); i++) 
	{
		result+=duration_tab(i);
	};
	return result;
}


/*!
    \fn CMainWindow::duration_tab(const int & index)
 */
long int CMainWindow::duration_tab(const int & index)
{
	long int result=0;
	QObjectList *objList=getSplitter(((QScrollView*)(tabwidget->page(index)))->viewport())->queryList("CTimerWidget",0,false,false);
	QObject *momEntry=NULL;

	// and for all CTimerWidgets (which are the grandchildren of the tabpage) store the data
	// in the document.
	for (  momEntry = objList->first(); momEntry; momEntry = objList->next() )
	{
		if (momEntry) 
		{
			result+=((CTimerWidget*)(momEntry))->calcInterval();
		}
	};
	
	return result;	
}

/*!
    \fn CTimerWidget::time_to_str(const long int & inp)
 */
QString CMainWindow::time_to_str(const long int & interval)
{
	QString momString="";
	momString.sprintf("%2ldD - %2ld:%2ld:%2ld\n",((interval)/86400),((interval)/3600)%24,((interval)/60)%60,(interval)%60);
	return momString;
}
