/***************************************************************************
 *   Copyright (C) 2004 by Thomas Thiel                                    *
 *   wolfchild@wolfchild.de                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qdatetime.h>
#include <qdom.h>
#include <qfile.h>
#include <qfiledialog.h>

#include <iostream>

using std::cout;
using std::cin;
using std::cerr;
using std::clog;
using std::endl;

#include "creportgenerator.h"
#include "CGenericConfig.h"

extern CGenericConfig *TimeTrackConf;


CReportGenerator::CReportGenerator(QWidget* parent, const char* name, WFlags fl)
    : ReportGenerator(parent,name,fl)
{
	setDateTimeStart(QDateTime::currentDateTime());
	setDateTimeStop(QDateTime::currentDateTime());
	if ((TimeTrackConf->existValue("TT_AOTemplate"))&(TimeTrackConf->getValue("TT_AOTemplate")!=""))
		loadFile(TimeTrackConf->getValue("TT_AOTemplate"));
	else resetDefaults_pressed();
}

CReportGenerator::~CReportGenerator()
{
}

/*$SPECIALIZATION$*/
void CReportGenerator::resetDefaults_pressed()
{
	setDocHeader("<HTML>%%n<HEAD>%%n%%a<TITLE>TimeTrack HTML Report %%x</TITLE>%%n</HEAD>%%n%%n<BODY>%%n%%a<H1>TimeTrack V1.1.3 HTML Report (%%y)</H1>%%n");
	setTabHeader("%%a<H2>Report for timers from group &quot;%%t&quot;</H2>%%n");
	setTimerHeader("%%a<P><B>Timer description</B>:</P>%%n%%a<P><PRE>%%d</PRE></P>%%n");
	setEntriesHeader("%%a<TABLE border=0 cellspacing=\"25px\">%%n%%a%%a<TR>%%n%%a%%a%%a<TH>Start</TH>%%n%%a%%a%%a<TH>Stop</TH>%%n%%a%%a%%a<TH>Description</TH>%%n%%a%%a</TR>%%n");
	setSingleEntry("%%a%%a<TR valign=\"top\">%%n%%a%%a%%a<TD>%%s</TD>%%n%%a%%a%%a<TD valign=\"top\">%%e</TD>%%n%%a%%a%%a<TD><PRE>%%c</PRE></TD>%%n%%a%%a</TR>%%n");
	setEntriesFooter("%%a</TABLE>%%n");
	setTimerFooter("%%a<hr>%%n");
	setTabFooter("%%a<P><sub>End of report for group %%t%%n</sub></P>");
	setDocFooter("%%a<P><sub>This report was generated by TimeTrack V1.1.3 (%%x)%%n</BODY>%%n</HTML>%%n</sub></P>");
}

void CReportGenerator::saveTemplate_pressed()
{
	// prepare a new DOM Document.
	QDomDocument doc( "XML" );
	QDomElement root = doc.createElement( "TimeTrack-ReportTemplate" );
	root.setAttribute("fileformat","1");
	doc.appendChild( root );

	QDomElement dom_docfooter = doc.createElement("docfooter");
	root.appendChild(dom_docfooter);
	QDomText t1 = doc.createTextNode( getDocFooter() );
	dom_docfooter.appendChild( t1 );

	QDomElement dom_docheader = doc.createElement("docheader");
	root.appendChild(dom_docheader);
	QDomText t2 = doc.createTextNode( getDocHeader() );
	dom_docheader.appendChild( t2 );

	QDomElement dom_entriesheader = doc.createElement("entriesheader");
	root.appendChild(dom_entriesheader);
	QDomText t3 = doc.createTextNode( getEntriesHeader() );
	dom_entriesheader.appendChild( t3 );

	QDomElement dom_entriesfooter = doc.createElement("entriesfooter");
	root.appendChild(dom_entriesfooter);
	QDomText t4 = doc.createTextNode( getEntriesFooter() );
	dom_entriesfooter.appendChild( t4 );

	QDomElement dom_singleentry = doc.createElement("singleentry");
	root.appendChild(dom_singleentry);
	QDomText t5 = doc.createTextNode( getSingleEntry() );
	dom_singleentry.appendChild( t5 );

	QDomElement dom_tabheader = doc.createElement("tabheader");
	root.appendChild(dom_tabheader);
	QDomText t6 = doc.createTextNode( getTabHeader() );
	dom_tabheader.appendChild( t6 );

	QDomElement dom_tabfooter = doc.createElement("tabfooter");
	root.appendChild(dom_tabfooter);
	QDomText t7 = doc.createTextNode( getTabFooter() );
	dom_tabfooter.appendChild( t7 );

	QDomElement dom_timerheader = doc.createElement("timerheader");
	root.appendChild(dom_timerheader);
	QDomText t8 = doc.createTextNode( getTimerHeader() );
	dom_timerheader.appendChild( t8 );

	QDomElement dom_timerfooter = doc.createElement("timerfooter");
	root.appendChild(dom_timerfooter);
	QDomText t9 = doc.createTextNode( getTimerFooter() );
	dom_timerfooter.appendChild( t9 );
	
	// get the filename.
	QString filename=QFileDialog::getSaveFileName(
                    QString::null,
                    "XML-Files (*.xml)",
                    this,
                    "TimeTrack File dialog",
                    "Choose a file to open" );
	
	if (filename=="") return;
	
	// prepare a file to save the xml data in.
	QFile outp_file( filename );
	
	if ( outp_file.open( IO_WriteOnly ) ) 
	{
		QTextStream txtstream( &outp_file );
		txtstream << doc.toString() << "\n";
		outp_file.close();
	};

	
}

void CReportGenerator::loadTemplate_pressed()
{
	// get Filename and check it
	QString filename=QFileDialog::getOpenFileName(
                    	QString::null,
                    	"XML-Files (*.xml)",
                    	this,
                    	"TimeTrack File dialog",
                    	"Choose a file to open" );

	if (filename=="") return;
	else loadFile(filename);
	
}

QString CReportGenerator::getDocFooter_substituted(const QString & duration_all,
						   const QString & filename)
{
	QString momString=getDocFooter();
	momString.replace("%%y",filename,true);
	momString.replace("%%x",QDateTime::currentDateTime().toString(Qt::LocalDate),true);
	momString.replace("%%o",duration_all,true);
	momString.replace("%%n","\n",true);
	momString.replace("%%a","\t",true);
	momString.replace("%%,",getDateTimeStart().toString(Qt::LocalDate),true);
	momString.replace("%%.",getDateTimeStop().toString(Qt::LocalDate),true);
	
	return momString;
}

QString CReportGenerator::getDocHeader_substituted(const QString & duration_all,
						   const QString & filename)
{
	QString momString=getDocHeader();
	momString.replace("%%y",filename,true);
	momString.replace("%%x",QDateTime::currentDateTime().toString(Qt::LocalDate),true);
	momString.replace("%%o",duration_all,true);
	momString.replace("%%n","\n",true);
	momString.replace("%%a","\t",true);
	momString.replace("%%,",getDateTimeStart().toString(Qt::LocalDate),true);
	momString.replace("%%.",getDateTimeStop().toString(Qt::LocalDate),true);

	return momString;
}

QString CReportGenerator::getTabFooter_substituted(const QString & tabname,                                           						   const QString & duration_tab,
						   const QString & duration_all,   
                                                   const QString & filename)
{
	QString momString=getTabFooter();
	momString.replace("%%t",tabname,true);
	momString.replace("%%y",filename,true);
	momString.replace("%%x",QDateTime::currentDateTime().toString(Qt::LocalDate),true);
	momString.replace("%%r",duration_tab,true);
	momString.replace("%%o",duration_all,true);
	momString.replace("%%n","\n",true);
	momString.replace("%%a","\t",true);
	momString.replace("%%,",getDateTimeStart().toString(Qt::LocalDate),true);
	momString.replace("%%.",getDateTimeStop().toString(Qt::LocalDate),true);

	return momString;
}

QString CReportGenerator::getTabHeader_substituted(const QString & tabname, 
						   const QString & duration_tab,
						   const QString & duration_all,   
                                                   const QString & filename)
{
	QString momString=getTabHeader();
	momString.replace("%%t",tabname,true);
	momString.replace("%%y",filename,true);
	momString.replace("%%x",QDateTime::currentDateTime().toString(Qt::LocalDate),true);
	momString.replace("%%r",duration_tab,true);
	momString.replace("%%o",duration_all,true);
	momString.replace("%%n","\n",true);
	momString.replace("%%a","\t",true);
	momString.replace("%%,",getDateTimeStart().toString(Qt::LocalDate),true);
	momString.replace("%%.",getDateTimeStop().toString(Qt::LocalDate),true);

	return momString;
}

QString CReportGenerator::getTimerFooter_substituted(const QString & timername, 
                                                     const QString & timerdesc,
						     const QString & tabname, 
						     const QString & duration,
						     const QString & duration_tab,
						     const QString & duration_all,   
						     const QString & filename)
{
	QString mom_desc=timerdesc;
	if (mom_desc=="") mom_desc="n/a";
	QString momString=getTimerFooter();
	momString.replace("%%i",timername,true);
	momString.replace("%%d",mom_desc,true);
	momString.replace("%%t",tabname,true);
	momString.replace("%%y",filename,true);
	momString.replace("%%x",QDateTime::currentDateTime().toString(Qt::LocalDate),true);
	momString.replace("%%u",duration,true);
	momString.replace("%%r",duration_tab,true);
	momString.replace("%%o",duration_all,true);
	momString.replace("%%n","\n",true);
	momString.replace("%%a","\t",true);
	momString.replace("%%,",getDateTimeStart().toString(Qt::LocalDate),true);
	momString.replace("%%.",getDateTimeStop().toString(Qt::LocalDate),true);
	
	return momString;
}

QString CReportGenerator::getTimerHeader_substituted(const QString & timername, 
                                                     const QString & timerdesc, 
						     const QString & tabname, 
						     const QString & duration,
						     const QString & duration_tab,
						     const QString & duration_all,   
						     const QString & filename)
{
	QString mom_desc=timerdesc;
	if (mom_desc=="") mom_desc="n/a";
	QString momString=getTimerHeader();
	momString.replace("%%i",timername,true);
	momString.replace("%%d",mom_desc,true);
	momString.replace("%%t",tabname,true);
	momString.replace("%%y",filename,true);
	momString.replace("%%x",QDateTime::currentDateTime().toString(Qt::LocalDate),true);
	momString.replace("%%u",duration,true);
	momString.replace("%%r",duration_tab,true);
	momString.replace("%%o",duration_all,true);
	momString.replace("%%n","\n",true);
	momString.replace("%%a","\t",true);
	momString.replace("%%,",getDateTimeStart().toString(Qt::LocalDate),true);
	momString.replace("%%.",getDateTimeStop().toString(Qt::LocalDate),true);

	return momString;
}

QString CReportGenerator::getEntriesHeader_substituted(const QString & timername, 
                                                       const QString & timerdesc,
						       const QString & tabname, 
						       const QString & duration,
						       const QString & duration_tab,
						       const QString & duration_all,   
						       const QString & filename)
{
	QString mom_desc=timerdesc;
	if (mom_desc=="") mom_desc="n/a";
	QString momString=getEntriesHeader();
	momString.replace("%%i",timername,true);
	momString.replace("%%d",mom_desc,true);
	momString.replace("%%t",tabname,true);
	momString.replace("%%y",filename,true);
	momString.replace("%%x",QDateTime::currentDateTime().toString(Qt::LocalDate),true);
	momString.replace("%%u",duration,true);
	momString.replace("%%r",duration_tab,true);
	momString.replace("%%o",duration_all,true);
	momString.replace("%%n","\n",true);
	momString.replace("%%a","\t",true);
	momString.replace("%%,",getDateTimeStart().toString(Qt::LocalDate),true);
	momString.replace("%%.",getDateTimeStop().toString(Qt::LocalDate),true);

	return momString;
}

QString CReportGenerator::getEntriesFooter_substituted(const QString & timername, 
                                                       const QString & timerdesc,
						       const QString & tabname, 
						       const QString & duration,
						       const QString & duration_tab,
						       const QString & duration_all,   
						       const QString & filename)
{
	QString mom_desc=timerdesc;
	if (mom_desc=="") mom_desc="n/a";
	QString momString=getEntriesFooter();
	momString.replace("%%i",timername,true);
	momString.replace("%%d",mom_desc,true);
	momString.replace("%%t",tabname,true);
	momString.replace("%%y",filename,true);
	momString.replace("%%x",QDateTime::currentDateTime().toString(Qt::LocalDate),true);
	momString.replace("%%u",duration,true);
	momString.replace("%%r",duration_tab,true);
	momString.replace("%%o",duration_all,true);
	momString.replace("%%n","\n",true);
	momString.replace("%%a","\t",true);
	momString.replace("%%,",getDateTimeStart().toString(Qt::LocalDate),true);
	momString.replace("%%.",getDateTimeStop().toString(Qt::LocalDate),true);

	return momString;
}

QString CReportGenerator::getSingleEntry_substituted(const QString & entrystart, 
                                                     const QString & entrystop, 
						     const QString & entrydesc, 
						     const QString & timername, 
						     const QString & timerdesc,
						     const QString & tabname, 
						     const QString & single_duration,
						     const QString & duration,
						     const QString & duration_tab,
						     const QString & duration_all,   
						     const QString & filename)
{
	QString mom_desc=timerdesc;
	if (mom_desc=="") mom_desc="n/a";
	QString momString=getSingleEntry();
	momString.replace("%%s",entrystart,true);
	momString.replace("%%e",entrystop,true);
	momString.replace("%%c",entrydesc,true);
	momString.replace("%%i",timername,true);
	momString.replace("%%d",mom_desc,true);
	momString.replace("%%t",tabname,true);
	momString.replace("%%y",filename,true);
	momString.replace("%%x",QDateTime::currentDateTime().toString(Qt::LocalDate),true);
	momString.replace("%%g",single_duration,true);
	momString.replace("%%u",duration,true);
	momString.replace("%%r",duration_tab,true);
	momString.replace("%%o",duration_all,true);
	momString.replace("%%n","\n",true);
	momString.replace("%%a","\t",true);
	momString.replace("%%,",getDateTimeStart().toString(Qt::LocalDate),true);
	momString.replace("%%.",getDateTimeStop().toString(Qt::LocalDate),true);

	return momString;
}



/*!
    \fn CReportGenerator::loadFile(const QString & filename)
 */
void CReportGenerator::loadFile(const QString & filename)
{
	// prepare a QDomDocument and read it from a file
	QDomDocument doc( "XML" );
    	QFile inp_file( filename );
    	
	if ( !inp_file.open( IO_ReadOnly ) )
	{
		cerr << "loadfile could not open file" << endl;
	        return;
	};
	
	// the xml parser wil put error data into these vars.
	QString errorMsg;
	int errorLine,errorCol;
    	
	if ( !doc.setContent( &inp_file, &errorMsg, &errorLine, &errorCol) ) 
	{
	        inp_file.close();
		cerr << "loadfile could not parse xml\nReason (Row:" << errorLine << "/Col:" << errorCol << "): \"" << errorMsg << "\"" << endl;
	        return;
    	};

	// the file has been parsed. Close it.
	inp_file.close();
	
	// get the root Node
	QDomElement root = doc.documentElement();
	
	// check if we have the correct file
	if ((root.tagName()=="TimeTrack-ReportTemplate")&&(root.attribute("fileformat","")=="1"))
	{
		QDomNodeList domList = root.elementsByTagName("docheader");
		if (domList.count()) 
		{
			QDomElement momElement = domList.item(0).toElement();
			setDocHeader(momElement.firstChild().toText().data());
		};
		
		domList = root.elementsByTagName("docfooter");
		if (domList.count()) 
		{
			QDomElement momElement = domList.item(0).toElement();
			setDocFooter(momElement.firstChild().toText().data());
		};
	
		domList = root.elementsByTagName("entriesfooter");
		if (domList.count()) 
		{
			QDomElement momElement = domList.item(0).toElement();
			setEntriesFooter(momElement.firstChild().toText().data());
		};
	
		domList = root.elementsByTagName("entriesheader");
		if (domList.count()) 
		{
			QDomElement momElement = domList.item(0).toElement();
			setEntriesHeader(momElement.firstChild().toText().data());
		};
	
		domList = root.elementsByTagName("singleentry");
		if (domList.count()) 
		{
			QDomElement momElement = domList.item(0).toElement();
			setSingleEntry(momElement.firstChild().toText().data());
		};
	
		domList = root.elementsByTagName("tabfooter");
		if (domList.count()) 
		{
			QDomElement momElement = domList.item(0).toElement();
			setTabFooter(momElement.firstChild().toText().data());
		};
	
		domList = root.elementsByTagName("tabheader");
		if (domList.count()) 
		{
			QDomElement momElement = domList.item(0).toElement();
			setTabHeader(momElement.firstChild().toText().data());
		};
	
		domList = root.elementsByTagName("timerfooter");
		if (domList.count()) 
		{
			QDomElement momElement = domList.item(0).toElement();
			setTimerFooter(momElement.firstChild().toText().data());
		};
	
		domList = root.elementsByTagName("timerheader");
		if (domList.count()) 
		{
			QDomElement momElement = domList.item(0).toElement();
			setTimerHeader(momElement.firstChild().toText().data());
		};
	
	};

}
