/***************************************************************************
 *   Copyright (C) 2004 by Thomas Thiel                                    *
 *   wolfchild@wolfchild.de                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "ctimereditdialog.h"
#include "ctimerentry.h"
#include <qdatetime.h>
#include <qlistview.h>
#include <qmessagebox.h>

// Main constructor. parses the EntryList and inserts the Entrys in the Listview.
CTimerEditDialog::CTimerEditDialog(QPtrList<CTimerEntry> *inp_EntryList, QWidget* parent, const char* name, bool modal, WFlags fl)
        : TimerEditDialog(parent,name, modal,fl)
{
	CTimerEntry *momEntry;
	EntryList=inp_EntryList;
	
	if (EntryList)
	{
		if (!EntryList->isEmpty())
		{
			for (  momEntry = EntryList->first(); momEntry; momEntry = EntryList->next() )
			{
				new QListViewItem(getEventListView(), momEntry->start.toString(Qt::LocalDate), momEntry->stop.toString(Qt::LocalDate), momEntry->description, QString::number(momEntry->id));
			};
			
			setEntryDesc(getEventListView()->firstChild()->text(2));
			setStartDate(EntryList->first()->start);
			setEndDate(EntryList->first()->stop);
		}
		else
		{
			setStartDate(QDateTime::currentDateTime());
			setEndDate(QDateTime::currentDateTime());
			setEntryDesc(" - enter new Description here and click \"add Entry\" - ");
		};
	};	

	stuff_has_changed_but_user_is_too_stupid_to_save_them_grrr=false;
	
	getEventListView()->setSelected(getEventListView()->firstChild(),true);
	
}

CTimerEditDialog::~CTimerEditDialog()
{
	// Qt doeas all the dirty stuff for us. We do absolut nottin'
}

/*$SPECIALIZATION$*/
// slot: called if the User wants to add a new entry.
void CTimerEditDialog::add_entry_pressed()
{
	// prepare a new CtimerEntry, fill it with the entered data and insert it into the EntryList.
	
	CTimerEntry *newEntry;
	QListViewItem *newItem;

	newEntry = new CTimerEntry();
	newEntry->start=getStartDate();
	newEntry->stop=getEndDate();
	newEntry->description=getEntryDesc();
	newEntry->id=nextFreeId();
		

	EntryList->append(newEntry);

	// now spawn a new entry in the Listview for it.
	newItem=new QListViewItem(getEventListView(), newEntry->start.toString(Qt::LocalDate), newEntry->stop.toString(Qt::LocalDate), newEntry->description, QString::number(newEntry->id));

	stuff_has_changed_but_user_is_too_stupid_to_save_them_grrr=false;

	// for it is the new one, we do select it for user convenience.
	getEventListView()->setSelected(newItem,true);
}

// called if the User wants to delete the active entry.
void CTimerEditDialog::delete_entry_pressed()
{
	// get the active item and check it for validity.
	QListViewItem *momItem=getEventListView()->selectedItem();
	CTimerEntry *momEntry;

	if (!momItem) return;

	// Let's check if the user lost his mind
	int result=QMessageBox::information(this,"TimeTrack Question","This will permanently delete this entry.\nAre you sure you want to do that?",QMessageBox::Yes|QMessageBox::Default,QMessageBox::No|QMessageBox::Escape);
		
	if (result==QMessageBox::No) return;

		
	// iterate all EntryList items and delete the correct one.
	for (  momEntry = EntryList->first(); momEntry; momEntry = EntryList->next() )
	{
		if ((momEntry->start.toString(Qt::LocalDate)==momItem->text(0))&&
		    (momEntry->stop.toString(Qt::LocalDate)==momItem->text(1))&&
		    (momEntry->description==momItem->text(2)))
		{
			EntryList->removeNode(EntryList->currentNode());
			delete momItem;
			momEntry=NULL;
		};
	};

	// if the List isn't empty, select the first entry for User convenience.
	if (getEventListView()->firstChild()) getEventListView()->setSelected(getEventListView()->firstChild(),true);


	stuff_has_changed_but_user_is_too_stupid_to_save_them_grrr=false;
}

// called if the user wants to change the current Entry.
void CTimerEditDialog::change_entry_pressed()
{
	// get the current ListViewItem and  check it for validity
	//QListViewItem *momItem=getEventListView()->selectedItem();
	CTimerEntry *momEntry;

	if (!lastItem) return;
		
	/// iterate the entryList
	for (  momEntry = EntryList->first(); momEntry; momEntry = EntryList->next() )
	{
		// If a EntryListItem matches the currently selected ListViewItem...
		if (momEntry->id==(QString(lastItem->text(3))).toInt())    
		{
			// ...change it.
			momEntry->description=getEntryDesc();
			momEntry->start=getStartDate();
			momEntry->stop=getEndDate();
			
			// afterwards correct the ListViewitem to the new values.
			lastItem->setText(0,momEntry->start.toString(Qt::LocalDate));
			lastItem->setText(1,momEntry->stop.toString(Qt::LocalDate));
			lastItem->setText(2,momEntry->description);

			// now close up and finish this function.
			stuff_has_changed_but_user_is_too_stupid_to_save_them_grrr=false;
			return;
		};

	};

}

// what to do, when the user changed the ListView Selection
void CTimerEditDialog::entrylist_currentChanged(QListViewItem *item)
{
	CTimerEntry *momEntry;

	if (stuff_has_changed_but_user_is_too_stupid_to_save_them_grrr)
	{
		int result=QMessageBox::information(this,"TimeTrack Question","The active entry has been changed but not saved yet. You will loose the changes if you don't click on \"change\". do you want to save the changes now ?",QMessageBox::Yes|QMessageBox::Default,QMessageBox::No|QMessageBox::Escape);
		
		if (result==QMessageBox::Yes) change_entry_pressed();
	}

	// if the new item is not defined, ...
	if (!item)
	{
		// set the widgets editors to default values
		setStartDate(QDateTime::currentDateTime());
		setEndDate(QDateTime::currentDateTime());
		setEntryDesc(" - enter new Description here and click \"add Entry\" - ");
		return;
	};

	// else insert the Items Data
	setEntryDesc(item->text(2));

	// find the corresponding CTimerentry Item and use the timer data from it to fill the widgets editor
	for (  momEntry = EntryList->first(); momEntry; momEntry = EntryList->next() )
	{
		if (momEntry->id==(QString(item->text(3))).toInt())    
		{
			setStartDate(momEntry->start);
			setEndDate(momEntry->stop);
		}
	};

	// of course the data has changed, we told the wdgets to. so reset the changed-status
	stuff_has_changed_but_user_is_too_stupid_to_save_them_grrr=false;

	// this one is tricky... As soon as the current item of the ListView was changed, item is not the current item anymore. But we need the current Item for saving purposes. so we hold back the lastItem. The lastItem is ALWAYS equal to the currentItem of the Listview, EXCEPT in this function, where it is the last chosen Item as long the function runs. here, at the end of the function, we change it to the current item.
	lastItem=item;
}


void CTimerEditDialog::close()
{
/*	if (stuff_has_changed_but_user_is_too_stupid_to_save_them_grrr)
	{
		int result=QMessageBox::information(this,"TimeTrack Question","The active entry has been changed but not been saved yet.\n\nDo you want to save it now ?",QMessageBox::Yes|QMessageBox::Default, QMessageBox::No|QMessageBox::Escape);

	      	if (result==QMessageBox::Yes) 
		{
			change_entry_pressed();
			TimerEditDialog::close();
		}
		else TimerEditDialog::close();
	}
	else TimerEditDialog::close();
*/
}

int CTimerEditDialog::nextFreeId()
{
	CTimerEntry *momEntry=NULL;
	int momId=-1;
	for (  momEntry = EntryList->first(); momEntry; momEntry = EntryList->next() )
	{
		if (momEntry->id>momId) momId=momEntry->id;
	};
	
	momId++;
	
	return momId;
}

         
