/***************************************************************************
 *   Copyright (C) 2004 by Thomas Thiel                                    *
 *   wolfchild@wolfchild.de                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <iostream>

using std::cout;
using std::cin;
using std::cerr;
using std::clog;
using std::endl;

#include <qlcdnumber.h>
#include <qinputdialog.h>

#include "ctimerwidget.h"
#include "ctimereditdialog.h"
#include "CGenericConfig.h"
#include <qmessagebox.h>
#include <qfiledialog.h>


extern CGenericConfig *TimeTrackConf;

// Main constructor. 
CTimerWidget::CTimerWidget(QWidget* parent, const char* name, WFlags fl)
: TimerWidget(parent,name,fl)
{
	// allocating a one second repetetive counter that calls the timersignal() slot to maintain the 
	// running counter
	// The Time in the EntryList is NOT depending on this counter. It is filled with timeStamps from QDateTime Objects.
	timer = new QTimer(this);
	connect(timer, SIGNAL(timeout()), this, SLOT(timersignal()));
	timer->start(1000,false);
	
	// initialize the attributes.
	// count stores if the timer is running (true) or not (false).
	count=false;
	// interval stores the timespan that has passed from the start til the current moment of a running timer.
	interval=0;
	
	// now, this is helpful. if the collection is deleted, the referenced Objects will be deleted, too, as long 
	// as this here is turned on.
	entryList.setAutoDelete(true);
	
	// prepare the outer cosmetics.
	setCaption("unnamed timer");
	set_lcdstring("---");
	set_data("--- inactive ---");
	tabname="New Tab";

	mainwindow=NULL;
}

CTimerWidget::CTimerWidget(CMainWindow* mainwin, QWidget* parent, const char* name, WFlags fl)
: TimerWidget(parent,name,fl)
{
	// allocating a one second repetetive counter that calls the timersignal() slot to maintain the 
	// running counter
	// The Time in the EntryList is NOT depending on this counter. It is filled with timeStamps from QDateTime Objects.
	timer = new QTimer(this);
	connect(timer, SIGNAL(timeout()), this, SLOT(timersignal()));
	timer->start(1000,false);
	
	// initialize the attributes.
	// count stores if the timer is running (true) or not (false).
	count=false;
	// interval stores the timespan that has passed from the start til the current moment of a running timer.
	interval=0;
	
	// now, this is helpful. if the collection is deleted, the referenced Objects will be deleted, too, as long 
	// as this here is turned on.
	entryList.setAutoDelete(true);
	
	// prepare the outer cosmetics.
	setCaption("unnamed timer");
	set_lcdstring("---");
	set_data("--- inactive ---");
	tabname="New Tab";

	mainwindow=mainwin;
	if (mainwindow!=NULL)
	{
		mainwindow->setCaption("TimeTrack - * "+mainwindow->currentFile);
		mainwindow->filesaveas->setEnabled(true);
		mainwindow->filesave->setEnabled(true);
	};
}

CTimerWidget::~CTimerWidget()
{
	// Nope, nothing to do here. Qt does all the dirty work for us.
	// Yes, it even takes care of the objects that are referenced through the entryList.
}

// controls what happens when the USer changes the action combobox
void CTimerWidget::action_changed(const QString& str)
{
	
	// if the user wants to rename the timer...
	if (str=="Rename Timer")
	{
		// ...let himn do it.
		bool ok_pressed=false;
		QString momString=QInputDialog::getText("TimeTrack", "Enter the new Name for the timer:", QLineEdit::Normal, getCaption(), &ok_pressed, this );
		if(ok_pressed) 
		{
			setCaption(momString);
			if (mainwindow!=NULL)
			{
		                mainwindow->setCaption("TimeTrack - * "+mainwindow->currentFile);
				mainwindow->filesaveas->setEnabled(true);
				mainwindow->filesave->setEnabled(true);
			};
		};
	};
	
	// if the User wants to reset the running timer...
	if ((str=="Reset Timer")&&(count))
	{
		// ... delete the unfinished CTimerEntry and prepare Widget for a new start.
		delete activeEntry;
		set_startstoplabel("Start Timer");
		set_data("--- timer was reset ---");
		count=!count;
		interval=calcInterval();
	};
	
	// if the user wants to edit the EntryList and the other widget data...
	if (str=="Edit Timer")
	{
		// ...spawn a CTimerEditDialog with our EntryList
		CTimerEditDialog *TimerEditDialog = new CTimerEditDialog(&entryList, this);
		
		// give it the necessary missing Data and let it run
		TimerEditDialog->setDesc(description);
		TimerEditDialog->setTimerName(getCaption());
		TimerEditDialog->exec();
		
		// then gather the changed data and actualize the interval var.
		description=TimerEditDialog->getDesc();
		setCaption(TimerEditDialog->getTimerName());
		interval=calcInterval();
	        if (mainwindow!=NULL)
       		{
	                mainwindow->setCaption("TimeTrack - * "+mainwindow->currentFile);
               		mainwindow->filesaveas->setEnabled(true);
                	mainwindow->filesave->setEnabled(true);
        	};
	};
	
	// if the user wants to remove this timer instance...
	if (str=="Remove Timer")
	{
	        if (mainwindow!=NULL)
       		{
	                mainwindow->setCaption("TimeTrack - * "+mainwindow->currentFile);
	                mainwindow->filesaveas->setEnabled(true);
	                mainwindow->filesave->setEnabled(true);
	        };

		// ...we choose a quick but painful death.
		delete this;
		return;
	};	
	
	// if the user wants to generate a html report for this timer...
	if (str=="Generate HTML report")
	{
		CReportGenerator reportgen(this,"reportgen");
		if (reportgen.exec()==QDialog::Accepted)
		{
			// .. we prepare a QString to store the generated html code in it.
			QString html_report=report(&reportgen,"n/a",true);

			QString report_filename = QFileDialog::getSaveFileName(
				QString::null,
				"HTML Files (*.htm *.html)",
				this,
				"TimeTrack save report dialog"
				"Choose a filename to save report under" );
			QFile file( report_filename );
			if ( file.open( IO_WriteOnly ) ) 
			{
				QTextStream stream( &file );
				stream << html_report;
				file.close();
				QMessageBox::information(this,"TimeTrack report generator","The report was successfully saved to file \""+report_filename+"\".",QMessageBox::Ok);
			}
			else QMessageBox::critical(this,"TimeTrack report generator","The report could NOT be successfully saved to file \""+report_filename+"\" !",QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
		};
	};	
	
	// reset the combobox to the default value.
	reset_action();

}

// called if the User clicks teh start/stop timer button.
void CTimerWidget::startstop_pressed()
{
	// invert the count status.
	count=!count;
	if (count)
	{
		// start was pressed, so spawn a new CTimerEntry and prepare it with default values
		activeEntry =  new CTimerEntry();
		activeEntry->id=nextFreeId();
		activeEntry->start=QDateTime::currentDateTime();
		activeEntry->stop=QDateTime::currentDateTime();
		activeEntry->description="n/a";
		set_startstoplabel("Stop Timer");
		
		if (TimeTrackConf->getValue("TT_DescMode")=="start")
		{
			bool ok_pressed;
			QString momString=QInputDialog::getText("TimeTrack", "Enter the description of the work you did in the last time interval:", QLineEdit::Normal, "New Entry", &ok_pressed, this );
			if (ok_pressed) activeEntry->description=momString;
				else activeEntry->description="--- n/a ---";
			set_data("Timer active (\""+activeEntry->description+"\")");
		}
		else
		{
			set_data("Timer active");
		};

		 
	}
	else 
	{
		// stop was pressed, so get the stop-timestamp and a description and put the CTimerentry into the EntryList.
		activeEntry->stop=QDateTime::currentDateTime();
		
		if (TimeTrackConf->getValue("TT_DescMode")=="end")
		{
			bool ok_pressed;
			QString momString=QInputDialog::getText("TimeTrack", "Enter the description of the work you did in the last time interval:", QLineEdit::Normal, "New Entry", &ok_pressed, this );
			if (ok_pressed) activeEntry->description=momString;
				else activeEntry->description="--- n/a ---";
		};
		
		set_startstoplabel("Start Timer");
		entryList.append(activeEntry);
		interval=calcInterval();
		
		set_data("Last Entry: \""+activeEntry->description+"\"");

	        if (mainwindow!=NULL)
	        {
	                mainwindow->setCaption("TimeTrack - * "+mainwindow->currentFile);
	                mainwindow->filesaveas->setEnabled(true);
	                mainwindow->filesave->setEnabled(true);
	                if (TimeTrackConf->getValue("TT_AutoSaveIfChanged")=="true") mainwindow->saveCurrent();
	        };
	};

}


/*!
    \fn CTimerWidget::timersignal()
 */
// called every single second by timer
void CTimerWidget::timersignal()
{
	// if the timer should count, increase interval.
	if (count)
	{
			 interval=calcInterval() + (QDateTime::currentDateTime().toTime_t() - activeEntry->start.toTime_t());
	}
	else interval=calcInterval();
	
	// Now prepare the string that is displayed in the LCDView.
	set_lcdstring(time_to_str(interval));

}

// called if the correct timerinterval has to be calculated.
long int CTimerWidget::calcInterval()
{
	long int counter=0;
	CTimerEntry *momEntry=NULL;

	// iterate all entries in the EntryList.
	for (  momEntry = entryList.first(); momEntry; momEntry = entryList.next() )
	{
		if (momEntry==NULL)
		{
			printf("CTimerEditdialog::calcInterval(): Interval calculation terminated: *momEntry == NULL\n\t\tInterval is inaccurate, EntryList is damaged.\n");
			return counter;
		};

		// add them aaaall up.
		counter+=momEntry->stop.toTime_t()-momEntry->start.toTime_t();

	};
	
	return counter;

};

// reads the CTimerwdiget Entry from a XML Document.
void CTimerWidget::readTimerData(const QDomElement & dom_input)
{
	// check if we are at home
	if (dom_input.tagName()!="CTimerWidget") return;

	// Make clean house
	entryList.clear();
	
	// read tabname and timername values from the top Level Node
	setCaption(dom_input.attribute("caption","unnamed Timer"));
	tabname=dom_input.attribute("tabname","New Tab");
	
	// Get the description Node...
	QDomNodeList domList = dom_input.elementsByTagName("TimerDescription");
	
	// ... and read it out.
	if (domList.count()) 
	{
		QDomElement momElement = domList.item(0).toElement();
		description=momElement.firstChild().toText().data();
	}
				
	// now get the List of Interval Entries.
	domList = dom_input.elementsByTagName("entryListItem");
	
	// for them all do construct a new CTimerEntry
	for( unsigned int i=0; i<domList.count();i++ )
	{
	
		CTimerEntry *momEntry=NULL;
		QString parser;

		momEntry=new CTimerEntry();
		
		QDomElement domElement=domList.item(i).toElement();
		momEntry->id=nextFreeId();
		momEntry->start.setTime_t(domElement.attribute("starttime","0").toLong());
		momEntry->stop.setTime_t(domElement.attribute("stoptime","0").toLong());
		
		QDomNodeList domListDesc = domElement.elementsByTagName("description");
		QDomElement momElement = domListDesc.item(0).toElement();
		momEntry->description=momElement.firstChild().toText().data();
		
		entryList.append(momEntry);
		
	};

	// now that we parsed the entrylist, we can check if there was a running timer saved
	// and prepare all the necessary steps to reinstate it.
		
	if (dom_input.hasAttribute("running_starttime"))
	{
		activeEntry =  new CTimerEntry();
		activeEntry->id=nextFreeId();
		activeEntry->start.setTime_t(dom_input.attribute("running_starttime","0").toLong());
		activeEntry->stop.setTime_t(dom_input.attribute("running_starttime","0").toLong());
		activeEntry->description=dom_input.attribute("running_description","n/a");
		set_startstoplabel("Stop Timer");
		
		if (TimeTrackConf->getValue("TT_DescMode")=="start")
			set_data("Timer active (\""+activeEntry->description+"\")");
		else
			set_data("Timer active");
		
		count=true;
	};
	
	// recalculate the timerInterval
	interval=calcInterval();

};

// prepares a DOM Node for writing in a XML File.
QDomElement CTimerWidget::writeTimerData()
{

	CTimerEntry *momEntry=NULL;    

	// We need a temporary document for DomNode GEneration.
	QDomDocument doc( "MomDoc" );
	QDomElement root = doc.createElement( "MomDoc" );
	doc.appendChild( root );

	// create a new top level node for this timer and write tabname and caption attributes
	QDomElement dom_twidget = doc.createElement( "CTimerWidget" );
	dom_twidget.setAttribute("caption",getCaption());
	dom_twidget.setAttribute("tabname",tabname);
	if (count)
	{
		dom_twidget.setAttribute("running_starttime",activeEntry->start.toTime_t());
		dom_twidget.setAttribute("running_description",activeEntry->description);
	};
	root.appendChild( dom_twidget );

	// now write the timer description Node and Text.
	QDomElement dom_twidget_desc = doc.createElement( "TimerDescription");
	dom_twidget.appendChild(dom_twidget_desc);
	
	QDomText t = doc.createTextNode( description );
	dom_twidget_desc.appendChild( t );

	// now iterate over all EntryList entries
	for (  momEntry = entryList.first(); momEntry; momEntry = entryList.next() )
	{
	   // check if the entry is kosher.
		if (momEntry==NULL)
		{
			cerr << "CTimerEditdialog::writetimerData(): XML Generation failed: *momEntry == NULL\n\t\tSave will be incomplete.\n" << endl;
			
		}
		else
		{
			// now insert a new Element in the DOM Tree, and insert start- and stoptime as attributes
			QDomElement dom_entry = doc.createElement( "entryListItem" );
			dom_entry.setAttribute("starttime",momEntry->start.toTime_t());
			dom_entry.setAttribute("stoptime",momEntry->stop.toTime_t());
			dom_twidget.appendChild( dom_entry );
			
			// then append the description Element and text.
			QDomElement dom_entry_desc = doc.createElement( "description" );
			dom_entry.appendChild( dom_entry_desc );
			QDomText dom_entry_desc_text = doc.createTextNode(momEntry->description);
			dom_entry_desc.appendChild( dom_entry_desc_text );
			
		};
		
	};
	
	// now return the generated Dom Tree
	return dom_twidget;
	
};

QString CTimerWidget::report(CReportGenerator *repgen, 
			     const QString & filename, 
			     long int duration_tab, 
			     long int duration_all, 
			     bool with_headers)
{
	// .. we prepare a QString to store the generated html code in it.
	QString html_report="";
	
	// save current interval exlicity to stop the function messing around with our entryList iterators
	int cur_interval=calcInterval();
	
	if (duration_all==0) duration_all=cur_interval;
	if (duration_tab==0) duration_tab=cur_interval;
	
	// We start with the html Header, if with_headers
	if (with_headers)
	{
		html_report+=repgen->getDocHeader_substituted(time_to_str(duration_all),filename);
	};
	
	// Now we start the main document. We begin with a headline and the decription.
	html_report+=repgen->getTimerHeader_substituted(getCaption(),description,tabname,time_to_str(cur_interval),time_to_str(duration_tab),time_to_str(duration_all),filename);

	if (entryList.isEmpty())
	{
		html_report+=repgen->getEntriesHeader_substituted(getCaption(),description,tabname,time_to_str(cur_interval),time_to_str(duration_tab),time_to_str(duration_all),filename);
		html_report+=repgen->getSingleEntry_substituted("n/a","n/a","This timer has no entries yet.",getCaption(),description,tabname,"n/a","n/a",time_to_str(duration_tab),time_to_str(duration_all),filename);
		html_report+=repgen->getEntriesFooter_substituted(getCaption(),description,tabname,time_to_str(cur_interval),time_to_str(duration_tab),time_to_str(duration_all),filename);
		html_report+=repgen->getTimerFooter_substituted(getCaption(),description,tabname,time_to_str(cur_interval),time_to_str(duration_tab),time_to_str(duration_all),filename);
		if (with_headers)
		html_report+=repgen->getDocFooter_substituted(time_to_str(duration_all),filename);
		
		return html_report;
	};

	
	// now prepare the table headers for the timer entries.
	html_report+=repgen->getEntriesHeader_substituted(getCaption(),description,tabname,time_to_str(cur_interval),time_to_str(duration_tab),time_to_str(duration_all),filename);

	// iterate all entries in the EntryList...
	CTimerEntry *momEntry=NULL;
	for (  momEntry = entryList.first(); momEntry; momEntry = entryList.next() )
	{
		if (momEntry!=NULL)
		{			
			if (repgen->isLimited())
			{
				bool showit=false;
				if ((repgen->isLimited_start())&(!repgen->isLimited_stop()))
				{
					if (momEntry->start>repgen->getDateTimeStart()) showit=true;
				}
				if ((!repgen->isLimited_start())&(repgen->isLimited_stop()))
				{
					if (momEntry->stop<repgen->getDateTimeStop()) showit=true;
				}
				if ((repgen->isLimited_start())&(repgen->isLimited_stop()))
				{
					if ((momEntry->stop<repgen->getDateTimeStop())&(momEntry->start>repgen->getDateTimeStart())) showit=true;
				}
				if ((!repgen->isLimited_start())&(!repgen->isLimited_stop())) showit=true;
				
				
				if (showit) html_report+=repgen->getSingleEntry_substituted(momEntry->start.toString(Qt::LocalDate),
		                                               momEntry->stop.toString(Qt::LocalDate),
							       momEntry->description,
							       getCaption(),description,tabname,time_to_str(momEntry->stop.toTime_t()-momEntry->start.toTime_t()),time_to_str(cur_interval),time_to_str(duration_tab),time_to_str(duration_all),filename);
			}
			else
			{
				// ...and put them in the report.
				html_report+=repgen->getSingleEntry_substituted(momEntry->start.toString(Qt::LocalDate),
		                                               momEntry->stop.toString(Qt::LocalDate),
							       momEntry->description,
							       getCaption(),description,tabname,time_to_str(momEntry->stop.toTime_t()-momEntry->start.toTime_t()),time_to_str(cur_interval),time_to_str(duration_tab),time_to_str(duration_all),filename);
			};
		};
	};
		
	// Close the table and (if with_headers) the document.
	html_report+=repgen->getEntriesFooter_substituted(getCaption(),description,tabname,time_to_str(cur_interval),time_to_str(duration_tab),time_to_str(duration_all),filename);
	html_report+=repgen->getTimerFooter_substituted(getCaption(),description,tabname,time_to_str(cur_interval),time_to_str(duration_tab),time_to_str(duration_all),filename);
	
	if (with_headers) html_report+=repgen->getDocFooter_substituted(time_to_str(duration_all),filename);
	
	return html_report;
	
}


/*!
    \fn CTimerWidget::time_to_str(const long int & inp)
 */
QString CTimerWidget::time_to_str(const long int & interval)
{
	QString momString="";
	momString.sprintf("%2ldD - %2ld:%2ld:%2ld\n",((interval)/86400),((interval)/3600)%24,((interval)/60)%60,(interval)%60);
	return momString;
}


int CTimerWidget::nextFreeId()
{
	CTimerEntry *momEntry=NULL;
	int momId=-1;
	for (  momEntry = entryList.first(); momEntry; momEntry = entryList.next() )
	{
		if (momEntry->id>momId) momId=momEntry->id;
	};
	
	momId++;
	
	return momId;
}
